from sdk32 import *

styles = ((BS_PUSHBUTTON,      'PUSHBUTTON'),
          (BS_DEFPUSHBUTTON,   'DEFPUSHBUTTON'),
          (BS_CHECKBOX,        'CHECKBOX'),
          (BS_AUTOCHECKBOX,    'AUTOCHECKBOX'),  
          (BS_RADIOBUTTON,     'RADIOBUTTON'),
          (BS_3STATE,          '3STATE'),
          (BS_AUTO3STATE,      'AUTO3STATE'),
          (BS_GROUPBOX,        'GROUPBOX'),
          (BS_USERBUTTON,      'USERBUTTON'),
          (BS_AUTORADIOBUTTON, 'AUTORADIOBUTTON'))

ps = PAINTSTRUCT()
rect = RECT()
tm = TEXTMETRIC()
x_char = y_char = 0

prm = 'wparam      LOWORD(lparam)  HIWORD(lparam)'
top = 'Control ID  Window Handle   Notification'
und = '__________  _____________   ____________'
fmt = ' %5u          %4x          %5u'

def wnd_proc(wnd, msg, wparam, lparam):
    global ps, rect, tm, x_char, y_char
    if msg == WM_CREATE:
        dc = GetDC(wnd)
        SelectObject(dc, GetStockObject(SYSTEM_FIXED_FONT))
        GetTextMetrics(dc, tm)
        x_char = tm.ave_char_width
        y_char = tm.height + tm.external_leading
        ReleaseDC(wnd, dc)
        i = 0
        for style, name in styles:
            CreateWindow('button', name,
                         WS_CHILD | WS_VISIBLE | style,
                         x_char, y_char * (1 + 2 * i),
                         20 * x_char, 7 * y_char / 4,
                         wnd, i,
                         GetModuleHandle(None), None)
            i += 1
    elif msg == WM_SIZE:
        rect.left = 24 * x_char
        rect.top = 3 * y_char
        rect.right = LOWORD(lparam)
        rect.bottom = HIWORD(lparam)
        return 0

    elif msg == WM_PAINT:
        InvalidateRect(wnd, rect, TRUE)

        dc = BeginPaint(wnd, ps)
        SelectObject(dc, GetStockObject(SYSTEM_FIXED_FONT))
        SetBkMode(dc, TRANSPARENT)
        TextOut(dc, 24 * x_char, 1 * y_char, prm, len(prm))
        TextOut(dc, 24 * x_char, 2 * y_char, top, len(top))
        TextOut(dc, 24 * x_char, 2 * y_char, und, len(und))
        EndPaint(wnd, ps)
        return 0

    elif msg == WM_COMMAND:
        ScrollWindow(wnd, 0, -y_char, rect, rect)
        dc = GetDC(wnd)
        SelectObject(dc, GetStockObject(SYSTEM_FIXED_FONT))
        text = fmt % (wparam, LOWORD(lparam), HIWORD(lparam))
        TextOut(dc, 24 * x_char, y_char * (rect.bottom / y_char - 1),
                text, len(text))
        ReleaseDC(wnd, dc)
        ValidateRect(wnd, None)
        return 0

    elif msg == WM_DESTROY:
        PostQuitMessage(0)
        return 0

    return DefWindowProc(wnd, msg, wparam, lparam)

def winmain(inst, prev_inst, cmd_line, cmd_show):
    app_name = 'BtnLook'

    wc = WNDCLASSEX()
    wc.style = CS_HREDRAW | CS_VREDRAW
    wc.wnd_proc = wnd_proc
    wc.inst = inst
    wc.cursor = LoadCursor(0, IDC_ARROW)
    wc.background = GetStockObject(WHITE_BRUSH)
    wc.name = app_name
    RegisterClassEx(wc)

    wnd = CreateWindow(app_name, 'Button Look', WS_OVERLAPPEDWINDOW,
                       CW_USEDEFAULT, CW_USEDEFAULT,
                       CW_USEDEFAULT, CW_USEDEFAULT, 0, 0, inst, 0)

    ShowWindow(wnd, cmd_show)
    UpdateWindow(wnd)

    msg = MSG()
    while 1:
        if not GetMessage(msg, 0, 0, 0):
            break
        TranslateMessage(msg)
        DispatchMessage(msg)
    return msg.wparam

if __name__ == '__main__':
    winmain(GetModuleHandle(None), 0, '', SW_SHOWNORMAL)
